"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function transformClassPropertyAssignmentToDecorator(_a) {
    var t = _a.types;
    return {
        name: require('./package').name,
        visitor: {
            // Logic loosely based on https://github.com/ember-cli/babel-plugin-ember-modules-api-polyfill
            ImportDeclaration: function (path, state) {
                var imports = state.opts.imports;
                var node = path.node;
                var importModuleName = node.source.value;
                var specifiers = path.get('specifiers');
                // This is all the imports to watch out for.
                var exportsToTransforms = imports[importModuleName];
                // Only walk specifiers if this is a module we need to watch out for.
                if (exportsToTransforms) {
                    // Filter all the specifiers whose usages we need to transform.
                    var importsToTransform = specifiers.filter(function (specifierPath) {
                        var specifier = specifierPath.node;
                        // We only care about these 2 specifiers.
                        if (specifier.type !== 'ImportDefaultSpecifier' &&
                            specifier.type !== 'ImportSpecifier') {
                            if (specifier.type === 'ImportNamespaceSpecifier') {
                                throw new Error("Using `import * as " + specifier.local + " from '" + importModuleName + "'` is not supported.");
                            }
                            return false;
                        }
                        // Determine the import name: either `default` or named.
                        var importName = specifier.type === 'ImportDefaultSpecifier'
                            ? 'default'
                            : specifier.imported.name;
                        // Skip, if export is not listed.
                        return exportsToTransforms.includes(importName);
                    });
                    if (importsToTransform.length === 0)
                        return;
                    for (var _i = 0, importsToTransform_1 = importsToTransform; _i < importsToTransform_1.length; _i++) {
                        var importSpecifier = importsToTransform_1[_i];
                        for (var _a = 0, _b = path.scope.bindings[importSpecifier.node.local.name].referencePaths; _a < _b.length; _a++) {
                            var referencePath = _b[_a];
                            var classPropertyPath = referencePath.findParent(function (path) { return !path.isExpression(); });
                            if (!classPropertyPath.isClassProperty())
                                throw referencePath.buildCodeFrameError("`" + importSpecifier.node.local.name + "` has to be a direct child of a `ClassProperty`.");
                            var classProperty = classPropertyPath.node;
                            classProperty.decorators = [t.decorator(classProperty.value)];
                            classProperty.value = null;
                        }
                    }
                }
            }
        }
    };
}
exports.default = transformClassPropertyAssignmentToDecorator;
